@@ -1,88 +1,60 @@
-"""
-Multiple points.
+"""Collections of points and related utilities
 """
 
-from ctypes import byref, c_double, c_int, c_void_p, cast, POINTER, pointer
+from ctypes import byref, c_double, c_void_p, cast, POINTER
+from ctypes import ArgumentError
 
 from shapely.geos import lgeos
-from shapely.geometry.base import BaseGeometry, GeometrySequence, exceptNull
+from shapely.geometry.base import BaseMultipartGeometry, exceptNull
 from shapely.geometry.point import Point, geos_point_from_py
 from shapely.geometry.proxy import CachingGeometryProxy
 
-
-def geos_multipoint_from_py(ob):
-    try:
-        # From array protocol
-        array = ob.__array_interface__
-        assert len(array['shape']) == 2
-        m = array['shape'][0]
-        n = array['shape'][1]
-        assert m >= 1
-        assert n == 2 or n == 3
-
-        # Make pointer to the coordinate array
-        cp = cast(array['data'][0], POINTER(c_double))
-
-        # Array of pointers to sub-geometries
-        subs = (c_void_p * m)()
-
-        for i in xrange(m):
-            geom, ndims = geos_point_from_py(cp[n*i:n*i+2])
-            subs[i] = cast(geom, c_void_p)
-
-    except AttributeError:
-        # Fall back on list
-        m = len(ob)
-        n = len(ob[0])
-        assert n == 2 or n == 3
-
-        # Array of pointers to point geometries
-        subs = (c_void_p * m)()
-        
-        # add to coordinate sequence
-        for i in xrange(m):
-            coords = ob[i]
-            geom, ndims = geos_point_from_py(coords)
-            subs[i] = cast(geom, c_void_p)
-            
-    return lgeos.GEOSGeom_createCollection(4, subs, m), n
+__all__ = ['MultiPoint', 'asMultiPoint']
 
 
-class MultiPoint(BaseGeometry):
+class MultiPoint(BaseMultipartGeometry):
 
-    """A multiple point geometry.
+    """A collection of one or more points
+
+    A MultiPoint has zero area and zero length.
+
+    Attributes
+    ----------
+    geoms : sequence
+        A sequence of Points
     """
 
-    def __init__(self, coordinates=None):
-        """Initialize.
-
+    def __init__(self, points=None):
+        """
         Parameters
         ----------
-        
-        coordinates : sequence or array
-            This may be an object that satisfies the numpy array protocol,
-            providing an M x 2 or M x 3 (with z) array, or it may be a sequence
-            of x, y (,z) coordinate sequences.
+        points : sequence
+            A sequence of (x, y [,z]) numeric coordinate pairs or triples or a
+            sequence of objects that implement the numpy array interface,
+            including instaces of Point.
 
         Example
         -------
+        Construct a 2 point collection
 
-        >>> geom = MultiPoint([[0.0, 0.0], [1.0, 2.0]])
-        >>> geom = MultiPoint(array([[0.0, 0.0], [1.0, 2.0]]))
-        
-        Each result in a line string from (0.0, 0.0) to (1.0, 2.0).
+          >>> ob = MultiPoint([[0.0, 0.0], [1.0, 2.0]])
+          >>> len(ob.geoms)
+          2
+          >>> type(ob.geoms[0]) == Point
+          True
         """
-        BaseGeometry.__init__(self)
+        super(MultiPoint, self).__init__()
 
-        if coordinates is None:
-            # allow creation of null lines, to support unpickling
+        if points is None:
+            # allow creation of empty multipoints, to support unpickling
             pass
         else:
-            self._geom, self._ndim = geos_multipoint_from_py(coordinates)
+            self._geom, self._ndim = geos_multipoint_from_py(points)
 
+    def shape_factory(self, *args):
+        return Point(*args)
 
     @property
-    @exceptNull
     def __geo_interface__(self):
         return {
             'type': 'MultiPoint',
@@ -119,31 +91,9 @@ class MultiPoint(BaseGeometry):
         return ai
     __array_interface__ = property(array_interface)
 
-    def _get_coords(self):
-        raise NotImplementedError, \
-        "Component rings have coordinate sequences, but the polygon does not"
-
-    def _set_coords(self, ob):
-        raise NotImplementedError, \
-        "Component rings have coordinate sequences, but the polygon does not"
-
-    @property
-    def coords(self):
-        raise NotImplementedError, \
-        "Multipart geometries do not themselves provide coordinate sequences"
-
-    @property
-    @exceptNull
-    def geoms(self):
-        return GeometrySequence(self, Point)
-        
 
 class MultiPointAdapter(CachingGeometryProxy, MultiPoint):
 
-    """Adapts a Python coordinate pair or a numpy array to the multipoint
-    interface.
-    """
-    
     context = None
     _owned = False
 
@@ -173,10 +123,53 @@ class MultiPointAdapter(CachingGeometryProxy, MultiPoint):
 
 
 def asMultiPoint(context):
-    """Factory for MultiPointAdapter instances."""
+    """Adapt a sequence of objects to the MultiPoint interface"""
     return MultiPointAdapter(context)
 
 
+def geos_multipoint_from_py(ob):
+    try:
+        # From array protocol
+        array = ob.__array_interface__
+        assert len(array['shape']) == 2
+        m = array['shape'][0]
+        n = array['shape'][1]
+        assert m >= 1
+        assert n == 2 or n == 3
+
+        # Make pointer to the coordinate array
+        try:
+            cp = cast(array['data'][0], POINTER(c_double))
+        except ArgumentError:
+            cp = array['data']
+
+        # Array of pointers to sub-geometries
+        subs = (c_void_p * m)()
+
+        for i in xrange(m):
+            geom, ndims = geos_point_from_py(cp[n*i:n*i+2])
+            subs[i] = cast(geom, c_void_p)
+
+    except AttributeError:
+        # Fall back on list
+        m = len(ob)
+        try:
+            n = len(ob[0])
+        except TypeError:
+            n = ob[0]._ndim
+        assert n == 2 or n == 3
+
+        # Array of pointers to point geometries
+        subs = (c_void_p * m)()
+        
+        # add to coordinate sequence
+        for i in xrange(m):
+            coords = ob[i]
+            geom, ndims = geos_point_from_py(coords)
+            subs[i] = cast(geom, c_void_p)
+            
+    return lgeos.GEOSGeom_createCollection(4, subs, m), n
+
 # Test runner
 def _test():
     import doctest
