@@ -1,5 +1,5 @@
 """
-Iterative forms of operations.
+Iterative forms of operations
 """
 
 from ctypes import c_char_p, c_size_t
@@ -24,7 +24,7 @@ class IterOp(object):
     
     def __call__(self, context, iterator, value=True):
         if context._geom is None:
-            raise ValueError, "Null geometry supports no operations"
+            raise ValueError("Null geometry supports no operations")
         for item in iterator:
             try:
                 this_geom, ob = item
@@ -32,11 +32,11 @@ class IterOp(object):
                 this_geom = item
                 ob = this_geom
             if not this_geom._geom:
-                raise ValueError, "Null geometry supports no operations"
+                raise ValueError("Null geometry supports no operations")
             retval = self.fn(context._geom, this_geom._geom)
             if retval == 2:
-                raise PredicateError, \
-                    "Failed to evaluate %s" % repr(self.fn)
+                raise PredicateError(
+                    "Failed to evaluate %s" % repr(self.fn))
             elif bool(retval) == value:
                 yield ob
 
@@ -50,3 +50,4 @@ within = IterOp(lgeos.GEOSWithin)
 contains = IterOp(lgeos.GEOSContains)
 overlaps = IterOp(lgeos.GEOSOverlaps)
 equals = IterOp(lgeos.GEOSEquals)
+
